part of 'exercise_execute_plan_add_bloc.dart';

@immutable
abstract class ExerciseExecutePlanAddEvent extends Equatable {
  const ExerciseExecutePlanAddEvent();

  @override
  List<Object> get props => [];
}

class ExerciseExecutePlanAddLoad extends ExerciseExecutePlanAddEvent {
  const ExerciseExecutePlanAddLoad();
}

class ExerciseExecutePlanAddChangeQuantity extends ExerciseExecutePlanAddEvent {
  final double quantity;
  const ExerciseExecutePlanAddChangeQuantity({required this.quantity});

  @override
  List<Object> get props => [quantity];
}

class ExerciseExecutePlanAddChangeUnitQuantity extends ExerciseExecutePlanAddEvent {
  final double quantity;
  const ExerciseExecutePlanAddChangeUnitQuantity({required this.quantity});

  @override
  List<Object> get props => [quantity];
}

class ExerciseExecutePlanAddSubmit extends ExerciseExecutePlanAddEvent {
  const ExerciseExecutePlanAddSubmit();
}
